require 'rollbar/rails'

module Logging
  module Appenders
    # Factory for the Papertrails appender.
    def self.papertrail(*args)
      str = '::Logging::Appenders::Rollbar needs a name as first argument.'
      fail ArgumentError, str if args.empty?
      ::Logging::Appenders::Papertrail.new(*args)
    end

    class Papertrail < ::Logging::Appender
      def initialize(name, opts = {})
        ::Logging.init unless ::Logging.initialized?

        @name    = name.to_s
        @closed  = false
        @filters = []
        @mutex   = ReentrantMutex.new

        self.layout   = opts.fetch(:layout, ::Logging::Layouts::Basic.new)
        self.level    = opts.fetch(:level, nil)
        self.encoding = opts.fetch(:encoding, self.encoding)
        self.filters  = opts.fetch(:filters, nil)

        if opts.fetch(:header, true)
          header = @layout.header

          unless header.nil? || header.empty?
            begin
              write(header)
            rescue StandardError => err
              ::Logging.log_internal_error(err)
            end
          end
        end

        ::Logging::Appenders[@name] = self
      end

      def close(*args)
        super(false)
      end

      private

      def write(event)
        is_log_event = event.instance_of?(::Logging::LogEvent)
        message = is_log_event ? layout.format(event) : event.to_s
        return if message.empty?

        exception = event.data || StandardError.new(message)
        severity =
          case Logging::LNAMES[event.level]
          when 'DEBUG', 'INFO'
            'info'
          when 'WARN'
            'warning'
          when 'ERROR', 'FATAL'
            'error'
          end

        message = JSON.parse(message).merge(severity: severity)
        logger.error(exception)
      end

      def logger
        @logger ||= ActiveSupport::TaggedLogging.new(
          RemoteSyslogLogger.new(
            'logs2.papertrailapp.com', 35682,
            program: 'rails-dev',
            local_hostname: 'optional_hostname'
          )
        )
      end
    end
  end
end